@@ -6,19 +6,8 @@ use strict;
 use Getopt::Long;
 use Pod::Usage;
 use FindBin qw($Bin);
-use Bio::ToolBox::data_helper qw(
-	generate_tim_data_structure
-	find_column_index
-	splice_data_structure
-);
-use Bio::ToolBox::db_helper qw(
-	open_db_connection
-	get_new_genome_list 
-);
-use Bio::ToolBox::file_helper qw(
-	load_tim_data_file 
-	write_tim_data_file 
-);
+use Bio::ToolBox::Data;
+use Bio::ToolBox::db_helper qw(open_db_connection);
 my $parallel;
 eval {
 	# check for parallel support
@@ -31,7 +20,7 @@ eval {
 	require Bio::DB::Sam;
 	$BAM_OK = 0;
 };
-my $VERSION = '1.18';
+my $VERSION = '1.20';
 
 print "\n This program will calculate observed & expected CpGs\n\n";
 
@@ -126,33 +115,25 @@ my $start_time = time;
 
 ### Prepare the database and main data structure
 my $db;
-my $data;
+my $Data;
 if ($infile) {
 	# an input file of regions is provided
-	$data = load_tim_data_file($infile) or 
+	$Data = Bio::ToolBox::Data->new(file => $infile) or 
 		die " unable to open input file '$infile'!\n";
 	
 	# check database
 	unless ($database) {
-		$database = $data->{'db'} or 
+		$database = $Data->database or 
 			die " no database or fasta file given! use --help for more information\n";
 	}
-	
-	# open database
-	$db = open_sequence_db() or 
-		die " unable to open database connection!\n";
 }
 else {
 	# make a new genome list based on the type of database we're using
 	if ($database) {
-		# first open the database
-		$db = open_db_connection($database) or 
-			die " unable to open database connection!\n";
-		
-		# get the list from the database
-		$data = get_new_genome_list(
-			'db'   => $db,
-			'win'  => $window,
+		$Data = Bio::ToolBox::Data->new(
+			'feature' => 'genome',
+			'db'      => $database,
+			'win'     => $window,
 		) or die " unable to generate genome window list!\n";
 	}
 	else {
@@ -161,16 +142,10 @@ else {
 	}
 }
 
-# check the database
-my $db_ref = ref $db;
-unless ($db_ref =~ /SeqFeature|Fasta|Fai/) {
-	die " unsupported database type $db_ref!\n";
-}
-
 # check whether it is worth doing parallel execution
 if ($cpu > 1) {
-	while ($cpu > 1 and $data->{'last_row'}/$cpu < 100) {
-		# I figure we need at least 100 lines in each fork split to make 
+	while ($cpu > 1 and ($Data->last_row / $cpu) < 200) {
+		# I figure we need at least 200 lines in each fork split to make 
 		# it worthwhile to do the split, otherwise, reduce the number of 
 		# splits to something more worthwhile
 		$cpu--;
@@ -216,7 +191,7 @@ sub parallel_execution {
 		#### In child ####
 	
 		# splice the data structure
-		splice_data_structure($data, $i, $cpu);
+		$Data->splice_data($i, $cpu);
 		
 		# re-open database objects to make them clone safe
 		# pass true to avoid cached database objects
@@ -226,8 +201,7 @@ sub parallel_execution {
 		process_regions();
 		
 		# write out result
-		my $success = write_tim_data_file(
-			'data'     => $data,
+		my $success = $Data->write_file(
 			'filename' => "$child_base_name.$i",
 			'gz'       => 0, # faster to write without compression
 		);
@@ -261,14 +235,13 @@ sub parallel_execution {
 sub single_execution {
 	
 	# execute
+	$db = open_sequence_db();
 	process_regions();
 	
 	# write the data file
-	my $written_file = write_tim_data_file(
-		# we will write a tim data file
-		# appropriate extensions and compression should be taken care of
-		'data'     => $data,
+	my $written_file = $Data->write_file(
 		'filename' => $outfile,
+		'gz'       => $gz,
 	);
 	if ($written_file) {
 		print " Wrote data file '$written_file' ";
@@ -279,112 +252,49 @@ sub single_execution {
 }
 
 
-sub identify_indices {
-	
-	# the indices to identify
-	my ($chrom, $start, $stop);
-	
-	# check obvious indices
-	if ($data->{'gff'}) {
-		$chrom = 0;
-		$start = 3;
-		$stop  = 4;
-	}
-	elsif ($data->{'bed'}) {
-		$chrom = 0;
-		$start = 1;
-		$stop  = 2;
-	}
-	elsif ($data->{'program'} eq $0) {
-		# genome data generated by this program
-		$chrom = 0;
-		$start = 1;
-		$stop  = 2;
-	}
-	else {
-		# custom tim data file
-		$chrom = find_column_index($data, '^chr|seq|ref|id');
-		$start = find_column_index($data, '^start');
-		$stop  = find_column_index($data, '^stop|end');
-	
-		# check
-		unless (defined $chrom and defined $start and defined $stop) {
-			die " unable to identify one or more coordinate column indices!\n";
-		}
-	}
-	
-	return ($chrom, $start, $stop);
-}
-
-
 sub process_regions {
 	
-	# Identify the indices
-	my ($chr_i, $start_i, $stop_i) = identify_indices();
-	
 	# Add new columns
 	# Fraction gc
-	my $fgc_i = $data->{'number_columns'};
-	$data->{$fgc_i} = {
-		'name'   => 'Fraction_GC',
-		'index'  => $fgc_i,
-	};
-	$data->{'number_columns'} += 1;
+	my $fgc_i = $Data->add_column('Fraction_GC');
 	
 	# number of CpG
-	my $cg_i = $data->{'number_columns'};
-	$data->{$cg_i} = {
-		'name'   => 'Number_CpG',
-		'index'  => $cg_i,
-	};
-	$data->{'number_columns'} += 1;
+	my $cg_i = $Data->add_column('Number_CpG');
 	
 	# expected number of CpG
-	my $exp_i = $data->{'number_columns'};
-	$data->{$exp_i} = {
-		'name'   => 'Expected_CpG',
-		'index'  => $exp_i,
-	};
-	$data->{'number_columns'} += 1;
+	my $exp_i = $Data->add_column('Expected_CpG');
 	
 	# observed/expected ratio
-	my $oe_i = $data->{'number_columns'};
-	$data->{$oe_i} = {
-		'name'   => 'Obs_Exp_Ratio',
-		'index'  => $oe_i,
-	};
-	$data->{'number_columns'} += 1;
-	
-	# add column header names
-	foreach ($fgc_i, $cg_i, $exp_i, $oe_i) { 
-		$data->{'data_table'}->[0][$_] = $data->{$_}{'name'};
-	}
+	my $oe_i = $Data->add_column('Obs_Exp_Ratio');
 	
 	
 	# Process the regions
-	for (my $row = 1; $row <= $data->{'last_row'}; $row++) {
+	$Data->iterate( sub {
 		
+		my $row = shift;
 		# get the region subsequence
+		# we are using our own database and not $Data's internal database connection
+		# since we might be using the faster Bio::DB::Sam::Fai module
 		my $seq = $db->seq(
-			$data->{'data_table'}->[$row][$chr_i],
-			$data->{'data_table'}->[$row][$start_i],
-			$data->{'data_table'}->[$row][$stop_i] + 1,
+			$row->seq_id,
+			$row->start,
+			$row->end + 1,
 			# we add 1 bp so that we can count CpG that cross a window border
-		);
+		) || undef;
 		unless ($seq) {
 			# this may happen if 0 or >1 chromosomes match the name
 			# or possibly coordinates are off the end, although I thought this was 
 			# checked by the db adaptor
-			warn "No sequence for segment " . 
-				$data->{'data_table'}->[$row][$chr_i] . ":" .
-				$data->{'data_table'}->[$row][$start_i] . ".." .
-				$data->{'data_table'}->[$row][$stop_i] . " at row $row, skipping.\n";
+			my $w = sprintf 
+				"No sequence available for segment %s:%s..%s at row %s, skipping\n", 
+				$row->seq_id, $row->start, $row->end, $row->row_index;
+			warn $w;
 			
 			# fill out null data
-			$data->{'data_table'}->[$row][$fgc_i] = '.';
-			$data->{'data_table'}->[$row][$cg_i]  = '.';
-			$data->{'data_table'}->[$row][$exp_i] = '.';
-			$data->{'data_table'}->[$row][$oe_i]  = '.';
+			$row->value($fgc_i, '.');
+			$row->value($cg_i, '.');
+			$row->value($exp_i, '.');
+			$row->value($oe_i, '.');
 			next;
 		}
 		
@@ -407,27 +317,26 @@ sub process_regions {
 		# we subtract 1 from the length because we added 1 when we generated the seq
 		if (length($seq) > 1) {
 			# must have reasonable length to avoid div by 0 errors
-			$data->{'data_table'}->[$row][$fgc_i] = 
-				sprintf "%.3f", ($numC + $numG) / (length($seq) - 1); # fraction GC
+			$row->value($fgc_i, 
+				sprintf "%.3f", ($numC + $numG) / (length($seq) - 1) );
 		
-			$data->{'data_table'}->[$row][$cg_i]  = $numCG; # number CpG
+			$row->value($cg_i, $numCG);
 		
-			$data->{'data_table'}->[$row][$exp_i] = 
-				sprintf "%.0f", ($numC * $numG) / (length($seq) - 1); # expected CpG
+			$row->value($exp_i,  
+				sprintf "%.0f", ($numC * $numG) / (length($seq) - 1) );
 		
-			$data->{'data_table'}->[$row][$oe_i]  = 
-				$data->{'data_table'}->[$row][$exp_i] ? # avoid div by 0
-				sprintf("%.3f", $numCG / $data->{'data_table'}->[$row][$exp_i]) : 
-				0; # obs/exp ratio
+			$row->value($oe_i, 
+				$row->value($exp_i) ? # avoid div by 0
+				sprintf("%.3f", $numCG / $row->value($exp_i) ) : 0);
 		}
 		else {
 			# a sequence of 1 bp? odd, just record default values
-			$data->{'data_table'}->[$row][$fgc_i] = 0;
-			$data->{'data_table'}->[$row][$cg_i]  = $numCG; # number CpG
-			$data->{'data_table'}->[$row][$exp_i] = 0;
-			$data->{'data_table'}->[$row][$oe_i]  = 0;
+			$row->value($fgc_i, 0);
+			$row->value($cg_i, $numCG); 
+			$row->value($exp_i, 0);
+			$row->value($oe_i, 0);
 		}
-	}
+	} );
 	
 }
 
@@ -467,6 +376,7 @@ CpG_calculator.pl --db <text> [--options...]
   
   Options:
   --db <name|file|directory>
+  --fasta <file|directory>
   --in <filename>
   --win <integer>
   --out <filename> 
@@ -483,6 +393,8 @@ The command line flags and descriptions:
 
 =item --db <name|file|directory>
 
+=item --fasta <file|directory>
+
 Provide the name of a Bio::DB::SeqFeature::Store database from which to 
 collect the genomic sequence. Alternatively, provide the name 
 of an uncompressed Fasta file (multi-fasta is ok) or directory containing 
