@@ -7,7 +7,6 @@ use Getopt::Long;
 use Pod::Usage;
 use Bio::ToolBox::data_helper qw(
 	find_column_index
-	format_with_commas
 );
 use Bio::ToolBox::file_helper qw(
 	open_tim_data_file
@@ -15,7 +14,8 @@ use Bio::ToolBox::file_helper qw(
 	open_to_write_fh
 	convert_genome_data_2_gff_data
 );
-my $VERSION = '1.15';
+use Bio::ToolBox::utility;
+my $VERSION = '1.20';
 
 print "\n This script will convert a data file to a GFF\n\n";
 
