@@ -1,4 +1,4 @@
-#!/usr/bin/env perl
+#!/usr/bin/perl
 
 # documentation at end of file
 
@@ -14,7 +14,7 @@ use Bio::ToolBox::file_helper qw(
 	open_tim_data_file
 	open_to_write_fh
 );
-my $VERSION = '1.14';
+my $VERSION = '1.15';
 
 print "\n This script will export a data file to a wig file\n\n";
 
@@ -915,12 +915,13 @@ The command line flags and descriptions:
 
 =item --in <filename>
 
-Specify the file name of a data file, with or without the --in flag.
-The file may be any tab-delimited text file, including GFF, BED,
-BedGraph, or SGR file formats. Genome coordinate columns should be
-recognizable or manually specified, including chromosome, start, and
-optionally stop. Strand is ignored in input data files. The file may 
-be compressed with gzip.
+Specify an input file containing either a list of database features or 
+genomic coordinates for which to collect data. The file should be a 
+tab-delimited text file, one row per feature, with columns representing 
+feature identifiers, attributes, coordinates, and/or data values. Genome 
+coordinates are required. The first row should be column headers. Text 
+files generated by other B<BioToolBox> scripts are acceptable. Files may 
+be gzipped compressed.
 
 =item --out <filename>
 
@@ -954,8 +955,8 @@ specifying --step=bed.
 
 Optionally define the step size in bp for 'fixedStep' wig file. This 
 value is automatically determined from the table's metadata, if available. 
-If the --step option is explicitly defined as 'fixed', then the step size 
-may also be explicitly defined. If this value is not explicitly
+If the C<--step> option is explicitly defined as 'fixed', then the step 
+size may also be explicitly defined. If this value is not explicitly
 defined or automatically determined, the variableStep format is used by
 default.
 
@@ -1010,7 +1011,7 @@ Do (not) include the track line at the beginning of the wig file. Wig
 files normally require a track line, but if you will be converting to 
 the binary bigwig format, the converter requires no track line. Why it 
 can't simply ignore the line is beyond me. This option is automatically 
-set to false when the --bigwig option is enabled.
+set to false when the C<--bigwig> option is enabled.
 
 =item --mid
 
@@ -1060,10 +1061,11 @@ Alternatively, provide a name of a database, below.
 
 =item --db <database>
 
-Specify the database from which chromosome lengths can be derived when 
-generating a bigwig file. This option is only required when generating 
-bigwig files. It may also be supplied from the metadata in the source 
-data file.
+Specify the name of a C<Bio::DB::SeqFeature::Store> annotation database 
+or other indexed data file, e.g. Bam or bigWig file, from which chromosome 
+length information may be obtained. For more information about using databases, 
+see L<https://code.google.com/p/biotoolbox/wiki/WorkingWithDatabases>. It 
+may be supplied from the input file metadata.
 
 =item --bwapp </path/to/wigToBigWig>
 
